import { NextRequest } from 'next/server';

interface Provider {
  handleCallback(request: NextRequest): Promise<Response>;
  handleSignIn(request: NextRequest, referer?: string): Promise<Response>;
  handleSignOut(request: NextRequest): Promise<void>;
  handleAuthCheck(token: string): Promise<{ user: UserType } | false>;
}

declare class Database {
  public createAccount(data: AccountType): Promise<AccountType>;
  public createUser(data: UserType): Promise<UserType>;
  public createSession(data: SessionType): Promise<SessionType>;
  public findAccount(data: Partial<AccountType>): Promise<AccountType | undefined>;
  public findUser(data: Partial<UserType>): Promise<UserType | undefined>;

  public findSession(data: Partial<SessionType>): Promise<SessionType | undefined>;
  public purgeSessions(user: UserType) : Promise<boolean>;
}


declare class Cache {
  public getValue(key: string): Promise<string | null>;
  public setValue(key: string, value: string, options?: { expire: number; }): Promise<unknown>;
  public deleteKey(key: string): Promise<number>;
}

interface SingleConfig {
  provider: Provider,
  database: Database,
  cache: Cache
}

interface Config {
  [key: string]: SingleConfig
}

type Secret = string

interface Session {
  user: UserType;
}

interface AccountType {
  accountId: string;
  provider: string;
  accessToken: string;
  refreshToken: string;
  expiresAt?: Date | undefined;
  tokenType?: string;
  scope: string;
}

interface SessionType {
  sessionToken: string;
  provider: string
  accountId: string
  expiresAt: Date
}

interface UserType {
  name: string;
  email: string;
  image: string;
  accountId: string;
  provider: string;
  emailVerified: boolean;
}

declare function sessionsInternal(): Promise<false | Session>;

type SignOutAndReloadResult = Promise<void>;

type ErrorResult = {
    success: false;
    message: string;
    status: number;
};
type JsonResult<T> = {
    success: true;
    data: T;
};

interface SignInConfig {
    redirect_url?: string;
    email?: string;
    password?: string;
    method?: string;
}
type SignInWithProviderResult = Promise<false | JsonResult<{
    url: string;
}> | ErrorResult>;

declare class Auth {
    static config: Config;
    static secret: Secret;
    static components: {
        signIn: (provider: keyof typeof Auth.config, config?: SignInConfig) => SignInWithProviderResult;
        signOut: () => SignOutAndReloadResult;
    };
    static routes: any;
    constructor(config: Config, secret: any);
}
declare const auth: typeof sessionsInternal;

export { Auth, auth };
